@@ -1,6 +1,11 @@
 """Collections of linestrings and related utilities
 """
 
+import sys
+
+if sys.version_info[0] < 3:
+    range = xrange
+
 from ctypes import c_double, c_void_p, cast, POINTER
 
 from shapely.geos import lgeos
@@ -40,7 +45,7 @@ class MultiLineString(BaseMultipartGeometry):
         """
         super(MultiLineString, self).__init__()
 
-        if lines is None:
+        if not lines:
             # allow creation of empty multilinestrings, to support unpickling
             pass
         else:
@@ -96,7 +101,7 @@ def geos_multilinestring_from_py(ob):
         # Array of pointers to sub-geometries
         subs = (c_void_p * L)()
 
-        for l in xrange(L):
+        for l in range(L):
             geom, ndims = geos_linestring_from_py(array['data'][l])
             subs[i] = cast(geom, c_void_p)
         N = lgeos.GEOSGeom_getDimensions(subs[0])
@@ -115,7 +120,7 @@ def geos_multilinestring_from_py(ob):
         subs = (c_void_p * L)()
         
         # add to coordinate sequence
-        for l in xrange(L):
+        for l in range(L):
             geom, ndims = geos_linestring_from_py(obs[l])
             subs[l] = cast(geom, c_void_p)
             
