@@ -10,7 +10,8 @@ try:
 except ImportError:
     import sys
     available = False
-    import_error_msg = tuple(sys.exc_info()[1])
+    # TODO: This does not appear to do anything useful
+    import_error_msg = sys.exc_info()[1]
 
 __all__ = ['available', 'enable', 'disable']
 _orig = {}
@@ -39,4 +40,4 @@ def disable():
     coords.CoordinateSequence.ctypes = _orig['CoordinateSequence.ctypes']
     linestring.geos_linestring_from_py = _orig['geos_linestring_from_py']
     polygon.geos_linearring_from_py = _orig['geos_linearring_from_py']
-    _orig.clear()
\ No newline at end of file
+    _orig.clear()
